local list = {}
local index = -1
local count = 0
local iname = 0

local tc,bl,tm
local initscript = "UO.CliNr=1"

----------------------------------------

local function ELocked(i)
	s = Ctrl.GetOutput(i)
	if s~="" then
		list[i+1].o.Lines.Add(s)
	end
	Ctrl.Unlock(i)
end

----------------------------------------

local function ELockTimer(sender)
	Ctrl.Lock(sender.Tag)
end

----------------------------------------

local function ETabChange()
	local i = tc.TabIndex
	if i>=0 and i<count then
		index = i
		list[i+1].o.BringToFront()
		list[i+1].v.BringToFront()
		local s = list[i+1].s
		s.BringToFront()
		s.SetFocus()
	end
end

----------------------------------------

local states = {"Stopped.","Running...","Paused."}
local function EStatusTimer()
	if count>0 then
		local s = Inst.ScriptMemo()
		local y = s.GetLineFromChar(s.SelStart)
		local x = s.SelStart-s.GetLineIndex(y)
		bl.Caption = y+1 .. "," .. x+1 .. " | " .. states[Ctrl.GetState(index)+1]
	else
		bl.Caption = ""
	end
end

----------------------------------------

function CreateInst(tabctrl,btmlbl)
	tc,bl = tabctrl,btmlbl
	tc.OnChange = ETabChange
	Ctrl.SetHandler(ELocked)
	Inst.New()

	tm = Obj.Create("TTimer")
	tm.OnTimer = EStatusTimer
	tm.Interval = 100
end

----------------------------------------

function FreeInst()
	Obj.Free(tm)
	Inst.CloseAll()
end

----------------------------------------
----------------------------------------
----------------------------------------

Inst = {}

function Inst.New()
	local s = Obj.Create("TMemo")
	s.WantTabs = true
	s.WordWrap = false
	s.ScrollBars = C.ssVertical
	s.Align = C.alClient
	s.PopupMenu = EditPopup --global!
	s.Parent = ScriptPanel

	local o = Obj.Create("TMemo")
	o.WantTabs = true
	o.WordWrap = false
	o.ScrollBars = C.ssVertical
	o.Align = C.alClient
	o.PopupMenu = EditPopup --global!
	o.Parent = OutputPanel

	local v = Obj.Create("TPanel")
	v.BevelOuter = 1
	v.BevelInner = 1
	v.Color = C.clWindow
	v.Align = C.alClient
	v.Parent = VarPanel

	local t = Obj.Create("TTimer")
	t.Tag = count
	t.OnTimer = ELockTimer
	t.Interval = 100

	index = count
	count = count + 1
	table.insert(list,{s=s,o=o,v=v,t=t})

	iname = iname + 1
	tc.Tabs.Add("new"..iname)
	tc.TabIndex = index

	if count==1 then
		tc.Show()
	end
	s.SetFocus()

	Ctrl.Open()
	Ctrl.SetPrefix(index,"$")
	Ctrl.SetName(index,"new"..iname)
end

----------------------------------------

function Inst.Open()
	local od = Obj.Create("TOpenDialog")
	od.DefaultExt = "lua"
	od.InitialDir = getinstalldir().."scripts"
	od.Filter = "Lua Files (*.lua)|*.lua|Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
	od.Options = 512 --filemustexist
	if od.Execute() then
		if count<1 then
			Inst.New()
		elseif Inst.ScriptMemo().Modified or
		Inst.ScriptMemo().Lines.Count>0 then
			Inst.New()
		end
		Ctrl.SetPrefix(index,"!")
		Ctrl.SetName(index,od.FileName)
		Inst.ScriptMemo().Lines.Text = LoadData(od.FileName)
	end
	Obj.Free(od)
end

----------------------------------------

function Inst.Close()
	if count>0 then
		Ctrl.Close(index)
		tc.Tabs.Delete(index)
		Obj.Free(list[index+1].s)
		Obj.Free(list[index+1].o)
		Obj.Free(list[index+1].v)
		Obj.Free(list[index+1].t)
		table.remove(list,index+1)
		count = count - 1

		index = index - 1
		if index<0 and count>0 then
			index = 0
		end
		tc.TabIndex = index
		ETabChange()

		if count==0 then
			tc.Hide()
		end
	end
end

----------------------------------------

function Inst.CloseAll()
	while count>0 do
		Inst.Close()
	end
end

----------------------------------------

function Inst.ScriptMemo()
	if count>0 then
		return list[index+1].s
	end
end

----------------------------------------

function Inst.OutputMemo()
	if count>0 then
		return list[index+1].o
	end
end

----------------------------------------

function Inst.VarPanel()
	if count>0 then
		return list[index+1].v
	end
end

----------------------------------------

function Inst.Start()
	if count>0 then
		if Ctrl.GetState(index)==0 then
			Inst.OutputMemo().Clear()
			Ctrl.SetScript(index,initscript.." "..Inst.ScriptMemo().Lines.Text)
		end
		Ctrl.SetState(index,1) --start
	end
end

----------------------------------------

function Inst.Pause()
	if count>0 then
		Ctrl.SetState(index,2) --pause
	end
end

----------------------------------------

function Inst.Stop()
	if count>0 then
		Ctrl.SetState(index,0) --stop
		Ctrl.SetReset(index,2) --reset asap
	end
end

----------------------------------------

local function GetActiveMemo()
	if list[index+1].o.Focused then
		return list[index+1].o
	end
	return list[index+1].s
end
function Inst.Cut()
	if count>0 then
		GetActiveMemo().CutToClipboard()
	end
end
function Inst.Copy()
	if count>0 then
		GetActiveMemo().CopyToClipboard()
	end
end
function Inst.Paste()
	if count>0 then
		GetActiveMemo().PasteFromClipboard()
	end
end
function Inst.Delete()
	if count>0 then
		GetActiveMemo().ClearSelection()
	end
end
function Inst.SelectAll()
	if count>0 then
		GetActiveMemo().SelectAll()
	end
end
function Inst.Undo()
	if count>0 then
		GetActiveMemo().Undo()
	end
end

----------------------------------------
